# encoding: utf-8
class Gw::WebmailFilter < ActiveRecord::Base
  include Sys::Model::Base
  include Sys::Model::Auth::Free

  NUMBER_OF_FILTERED_UIDS = 256

  belongs_to :status,
             :foreign_key => :state,
             :class_name => 'Sys::Base::Status'

  has_many :conditions,
            ->{ order(:sort_no) },
           foreign_key: :filter_id,
           class_name: 'Gw::WebmailFilterCondition',
           dependent: :destroy
           
  attr_accessor :in_columns, :in_inclusions, :in_values, :include_sub
  attr_reader :applied

  validates_presence_of :user_id, :state, :name, :conditions_chain, :action
  validates_numericality_of :sort_no, :greater_than_or_equal_to => 0, :only_integer => true

  validate :validate_conditions
  validate :validate_mailbox
  validate :validate_name

  after_save :save_conditions

  def readable
    where(user_id: Core.user.id)
  end

  def readable?
    user_id == Core.user.id
  end

  def editable?
    user_id == Core.user.id
  end

  def deletable?
    user_id == Core.user.id
  end

  def in_columns
    @in_columns = conditions.collect{|c| c.column} if @in_columns.nil?
    @in_columns
  end

  def in_inclusions
    @in_inclusions = conditions.collect{|c| c.inclusion} if @in_inclusions.nil?
    @in_inclusions
  end

  def in_values
    @in_values = conditions.collect{|c| c.value} if @in_values.nil?
    @in_values
  end

  def states
    [[I18n.t("helpers.enabled"),'enabled'],[I18n.t("helpers.disabled"),'disabled']]
  end

  def action_labels
    [[I18n.t("rumi.filter.view.column.action.move"), "move"],[I18n.t("rumi.filter.view.column.action.delete"),"delete"]]
  end

  def state_label
    states.each {|a| return a[0] if state == a[1].to_s }
    nil
  end

  def action_label
    action_labels.each {|a| return a[0] if action == a[1].to_s }
    nil
  end

  def mailbox_name
    Gw::WebmailMailbox.name_to_title(mailbox).gsub('.', '/')
  end

  def conditions_chain_labels
    [[I18n.t("rumi.filter.view.column.conditions.and"), "and"],[I18n.t("rumi.filter.view.column.conditions.or"),"or"]]
  end

  def conditions_chain_label
    conditions_chain_labels.each {|a| return a[0] if conditions_chain == a[1].to_s }
    nil
  end

  def condition_column_labels
    [[I18n.t("rumi.mails.view.column.subject_name") + "（Subject）","subject"],[I18n.t("rumi.mails.view.column.mail_list.from") + "（From）","from"],[I18n.t("rumi.mails.view.column.mail_list.to") + "（To）","to"]]
  end

  def condition_inclusion_labels
    [[I18n.t("rumi.filter.view.column.conditions.inclusion1"),"<"],[I18n.t("rumi.filter.view.column.conditions.inclusion2"),"!<"],[I18n.t("rumi.filter.view.column.conditions.inclusion3"),"=="],[I18n.t("rumi.filter.view.column.conditions.inclusion4"),"=~"]]
  end

  def apply(params)
    @applied = 0
    applied_uids = []
    idx = 0
    params[:filter] = self
    uids = Gw::WebmailMail.find_uid(:all, :select => params[:select], :conditions => params[:conditions])
    while (filtered = uids[idx, NUMBER_OF_FILTERED_UIDS]) && filtered.size > 0
      params[:timeout].check
      applied_uids += self.class.apply_uids(filtered, params)
      idx += filtered.size
    end
    @applied = applied_uids.size

    starred_uids = Gw::WebmailMailNode.find_ref_nodes(params[:select], applied_uids).map{|x| x.uid}
    Core.imap.select('Star')
    num = Core.imap.uid_store(starred_uids, "+FLAGS", [:Deleted]).size rescue 0
    Core.imap.expunge
    if num > 0
      Gw::WebmailMailNode.delete_nodes('Star', starred_uids)
    end

    return @applied
  end

  def self.apply_recents
    cond = {:user_id => Core.current_user.id, :state => 'enabled'}
    filters = Gw::WebmailFilter.where(cond).order("sort_no, id")
    cond = {:user_id => Core.current_user.id, :name => 'last_uid'}
    st   = Gw::WebmailSetting.where(cond).first || Gw::WebmailSetting.new(cond)
    next_uid = Core.imap.status('INBOX', ["UIDNEXT"])["UIDNEXT"]
    last_uid = (next_uid > 1) ? next_uid - 1 : 1
    imap_cnd = st.value.blank? ? ['RECENT'] : ['UID', "#{st.value.to_i + 1}:#{last_uid}", 'UNSEEN']
    error = nil
    begin
      if filters.size > 0
        idx = 0
        timeout = Sys::Lib::Timeout.new(60)
        uids = Gw::WebmailMail.find_uid(:all, :select => 'INBOX', :conditions => imap_cnd)
        while (filtered = uids[idx, NUMBER_OF_FILTERED_UIDS]) && filtered.size > 0
          timeout.check
          apply_uids(filtered, :select => 'INBOX', :filters => filters)
          idx += filtered.size
        end
      end
    rescue Sys::Lib::Timeout::Error => ex
      error = ex
    end

    if last_uid.to_s != st.value.to_s
      st.value = last_uid
      st.save(:validate => false)
      return last_uid, true, error
    end
    return last_uid, false, error
  end

  def self.apply_uids(uids, params)
    applied_uids = []
    matched = []
    mails = Gw::WebmailMail.fetch_for_filter(uids, params[:select])

    filters = params[:filters] || [params[:filter]]
    filters.each {|filter| matched << {:filter => filter, :uids => []}}

    mails.each do |mail|
      filters.each_with_index do |filter, idx|
        filter_matched = false
        begin
          filter.conditions.each do |c|
            values = []
            case c.column
            when 'subject'
              values += [mail.subject]
            when 'from'
              values += [ mail.friendly_from_addr ]
            when 'to'
              values += mail.friendly_to_addrs
            end

            syntax = nil
            case c.inclusion
            when '<'
              syntax = '!( value =~ /#{Regexp.quote(c.value)}/i ).nil?'
            when '!<'
              syntax = '( value =~ /#{Regexp.quote(c.value)}/i ).nil?'
            when '=='
              syntax = '( value == c.value )'
            when '=~'
              syntax = '( value.to_s =~ /#{c.value}/im )'
            end
            next unless syntax

            m = false
            values.each do |value|
              if eval(syntax)
                m = true
                break
              end
            end

            if m == true
              filter_matched = true
              break if filter.conditions_chain == 'or'
            else
              filter_matched = false
              break if filter.conditions_chain == 'and'
            end
          end #/filter.conditions

          if filter_matched
            matched[idx][:uids] << mail.uid
            break
          end
        rescue => e
          error_log(e)
        end
      end #/filters
    end #/mails

    matched.each do |m|
      next unless m[:uids].size > 0
      begin
        Gw::WebmailMailNode.delete_nodes(params[:select], m[:uids]) if params[:delete_cache]
        case m[:filter].action
        when "move"
          if Gw::WebmailMail.move_all(params[:select], m[:filter].mailbox, m[:uids])
            applied_uids += m[:uids]
          end
        when "delete"
          if Gw::WebmailMail.delete_all(params[:select], m[:uids])
            applied_uids += m[:uids]
          end
        end
      rescue => e
        error_log(e)
        next
      end
    end

    return applied_uids
  rescue => e
    error_log(e)
    return []
  end

  def last_condition
    Gw::WebmailFilterCondition.where(:filter_id => id).order('sort_no DESC').first
  end

  def self.register_spams(items)
    cond = {
      :user_id   => Core.current_user.id,
      :name      => "* " + I18n.t("rumi.filter.spam_mail"),
    }
    unless filter = self.where(cond).first
      filter = self.new(cond)
      filter.state = 'enabled'
      filter.sort_no = 0
      filter.conditions_chain = 'or'
      filter.action = 'delete'
      filter.mailbox = ''
      filter.save(:validate => false)
    end

    last_condition = filter.last_condition
    next_sort_no = last_condition ? last_condition.sort_no + 1 : 0

    items.each_with_index do |item, i|
      cond = {
        :user_id   => Core.current_user.id,
        :filter_id => filter.id,
        :column    => 'from',
        :inclusion => '<',
        :value     => item.from_addr
      }
      unless Gw::WebmailFilterCondition.where(cond).first
        cond[:sort_no] = next_sort_no + i
        fcond = Gw::WebmailFilterCondition.new(cond)
        fcond.save(:validate => false)
      end
    end

    spam_max_count = Enisys.config.application['webmail.filter_condition_max_count']

    cond = {:user_id => Core.current_user.id, :filter_id => filter.id}
    spam_count = Gw::WebmailFilterCondition.where(cond).count(:all)
    if spam_count > spam_max_count
      del_items = Gw::WebmailFilterCondition.where(cond).limit(spam_count - spam_max_count).order('sort_no')
      Gw::WebmailFilterCondition.where(:id => del_items.map{|item| item.id}).delete_all
    end
  end
private

protected
  def validate_mailbox
    return true if !mailbox.blank?
    return true if action !~ /^(move)$/
    errors.add :mailbox, :empty
  end

  def validate_name
    if name_changed?
      if (name_was == nil || name_was =~ /^[^*]/) && name =~ /^[*]/
        errors.add :name, I18n.t("helpers.message.valid_filter_name")
        name.gsub!(/^[*]+/, '')
      end
    end
  end

  def in_conditions
    #dummy
  end

  def validate_conditions
    if "#{in_columns.values.join}#{in_inclusions.values.join}#{in_values.values.join}".blank?
      return errors.add(:in_conditions, :empty)
    end
    in_columns.each do |i, c|
      next if "#{c}#{in_inclusions[i]}#{in_values[i]}".blank?
      if c.blank? || in_inclusions[i].blank? || in_values[i].blank?
        return errors.add(:in_conditions, :invalid)
      end
      if in_inclusions[i] == '=~'
        begin
          timeout(1) { test = ("text" =~ /#{in_values[i]}/im) }
        rescue => e
          errors.add(:in_conditions, I18n.t("helpers.message.valid_regex", value: in_values[i]))
        end
      end
    end
  end

  def save_conditions
    return true if !in_columns.is_a?(Hash) && !in_columns.is_a?(Array)

    in_conds = []
    in_columns.each do |i, c|
      next if c.blank? || in_inclusions[i].blank? || in_values[i].blank?
      in_conds << {column: c, inclusion: in_inclusions[i], value: in_values[i]}
    end

    #紐づいているfiler_idのレコードを消去
    conditions.delete_all

    in_conds.each_with_index do |c, idx|
      new_filter = Gw::WebmailFilterCondition.new
      new_filter.attributes = {
        :user_id   => user_id,
        :filter_id => id,
        :sort_no   => idx,
        :column    => c[:column],
        :inclusion => c[:inclusion],
        :value     => c[:value]
      }
      new_filter.save
    end
    conditions(true)
    true
  end

end
